#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: system_issue_definitions_info
short_description: Information module for System Issue Definitions Info
description:
- This module represents an alias of the module system_issue_definitions_v1_info
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deviceType:
    description:
    - >
      DeviceType query parameter. These are the device families/types supported for system issue definitions. If
      no input is made on device type, all device types are considered.
    type: str
  profileId:
    description:
    - >
      ProfileId query parameter. The profile identier to fetch the profile associated issue defintions. The
      default is `global`. Please refer Network design profiles documentation for more details.
    type: str
  id:
    description:
    - >
      Id query parameter. The definition identifier. Examples id=015d9cba-4f53-4087-8317-7e49e5ffef46 (single
      entity id request) id=015d9cba-4f53-4087-8317-7e49e5ffef46&id=015d9cba-4f53-4087-8317-7e49e5ffef47 (multiple
      ids in the query param).
    type: str
  name:
    description:
    - >
      Name query parameter. The list of system defined issue names. (Ex."BGP_Down") Examples name=BGP_Down (single
      entity uuid requested) name=BGP_Down&name=BGP_Flap (multiple issue names separated by & operator).
    type: str
  priority:
    description:
    - >
      Priority query parameter. Issue priority, possible values are P1, P2, P3, P4. `P1` A critical issue that
      needs immediate attention and can have a wide impact on network operations. `P2` A major issue that can
      potentially impact multiple devices or clients. `P3` A minor issue that has a localized or minimal impact.
      `P4` A warning issue that may not be an immediate problem but addressing it can optimize the network
      performance.
    type: str
  issueEnabled:
    description:
    - IssueEnabled query parameter. The enablement status of the issue definition, either true or false.
    type: bool
  attribute:
    description:
    - >
      Attribute query parameter. These are the attributes supported in system issue definitions response. By
      default, all properties are sent in response.
    type: str
  offset:
    description:
    - >
      Offset query parameter. Specifies the starting point within all records returned by the API. It's one based
      offset. The starting value is 1.
    type: float
  limit:
    description:
    - Limit query parameter. Maximum number of records to return.
    type: float
  sortBy:
    description:
    - SortBy query parameter. A field within the response to sort by.
    type: str
  order:
    description:
    - Order query parameter. The sort order of the field ascending or descending.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Issues ReturnsAllIssueTriggerDefinitionsForGivenFiltersV1
  description: Complete reference of the ReturnsAllIssueTriggerDefinitionsForGivenFiltersV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!returns-all-issue-trigger-definitions-for-given-filters
notes:
  - SDK Method used are
    issues.Issues.returns_all_issue_trigger_definitions_for_given_filters_v1,

  - Paths used are
    get /dna/intent/api/v1/systemIssueDefinitions,
  - It should be noted that this module is an alias of system_issue_definitions_v1_info

"""

EXAMPLES = r"""
- name: Get all System Issue Definitions Info
  cisco.dnac.system_issue_definitions_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    deviceType: string
    profileId: string
    id: string
    name: string
    priority: string
    issueEnabled: True
    attribute: string
    offset: 0
    limit: 0
    sortBy: string
    order: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "id": "string",
        "name": "string",
        "displayName": "string",
        "description": "string",
        "priority": "string",
        "defaultPriority": "string",
        "deviceType": "string",
        "issueEnabled": true,
        "profileId": "string",
        "definitionStatus": "string",
        "categoryName": "string",
        "synchronizeToHealthThreshold": true,
        "thresholdValue": 0,
        "lastModified": "string"
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_key_info
version_added: 6.4.0
short_description: Gather information about EC2 key pairs in AWS
description:
    - Gather information about EC2 key pairs in AWS.
author:
  - Aubin Bikouo (@abikouo)
options:
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value. See
        U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeKeyPairs.html) for possible filters. Filter
        names and values are case sensitive.
    required: false
    default: {}
    type: dict
  names:
    description:
      - The key pair names.
    required: false
    type: list
    elements: str
    default: []
  ids:
    description:
      - The IDs of the key pairs.
    required: false
    type: list
    elements: str
    default: []
  include_public_key:
    description:
      - Whether or not to include the public key material in the response.
    type: bool
    default: false
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Gather information about all key pairs
  amazon.aws.ec2_key_info:

- name: Gather information about a specific key pair
  amazon.aws.ec2_key_info:
    names:
      - my-sample-key

- name: Retrieve EC2 key pair by fingerprint
  amazon.aws.ec2_key_info:
    filters:
      fingerprint: "1bSd8jVye3In5oF4zZI4o8BcXfdbYN+daCt9O1fh3Qk="
"""

RETURN = r"""
keypairs:
    description: A list of ec2 key pairs.
    returned: always
    type: complex
    contains:
        key_pair_id:
            description: The ID of the key pair.
            returned: always
            type: str
            sample: key-01238eb03f07d7268
        key_fingerprint:
            description: Fingerprint of the key.
            returned: always
            type: str
            sample: '05:97:1a:2a:df:f6:06:a9:98:4b:ca:05:71:a1:81:e8:ff:6d:d2:a3'
        key_name:
            description: The name of the key pair.
            returned: always
            type: str
            sample: my-sample-keypair
        key_type:
            description: The type of key pair.
            returned: always
            type: str
            sample: rsa
        public_key:
            description: The public key material.
            returned: always
            type: str
        create_time:
            description: The time the key pair was created.
            returned: always
            type: str
            sample: "2023-08-16T10:13:33.025000+00:00"
        tags:
            description: A dictionary representing the tags attached to the key pair.
            returned: always
            type: dict
            sample: '{"my_key": "my value"}'
"""


try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_key_pairs
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


def list_ec2_key_pairs(connection, module):
    ids = module.params.get("ids")
    names = module.params.get("names")
    include_public_key = module.params.get("include_public_key")
    filters = module.params.get("filters")
    if filters:
        filters = ansible_dict_to_boto3_filter_list(filters)

    params = {}
    if filters:
        params["Filters"] = filters
    if ids:
        params["KeyPairIds"] = ids
    if names:
        params["KeyNames"] = names
    if include_public_key:
        params["IncludePublicKey"] = True

    try:
        result = describe_key_pairs(connection, **params)
    except AnsibleEC2Error as e:
        module.fail_json_aws(e, msg="Failed to list EC2 key pairs")

    # Turn the boto3 result in to ansible_friendly_snaked_names
    snaked_keys = [camel_dict_to_snake_dict(key) for key in result]

    # Turn the boto3 result in to ansible friendly tag dictionary
    for key in snaked_keys:
        key["tags"] = boto3_tag_list_to_ansible_dict(key.get("tags", []), "key", "value")

    module.exit_json(keypairs=snaked_keys)


def main():
    argument_spec = dict(
        filters=dict(type="dict", default={}),
        names=dict(type="list", elements="str", default=[]),
        ids=dict(type="list", elements="str", default=[]),
        include_public_key=dict(type="bool", default=False),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    try:
        connection = module.client("ec2")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    list_ec2_key_pairs(connection, module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_fabric_border_device_v1
short_description: Resource module for Sda Fabric Border Device V1
description:
- Manage operations create and delete of the resource Sda Fabric Border Device V1.
- Add border device in SDA Fabric.
- Delete border device from SDA Fabric.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  deviceManagementIpAddress:
    description: DeviceManagementIpAddress query parameter.
    type: str
    version_added: 4.0.0
  payload:
    description: Sda Fabric Border Device's payload.
    elements: dict
    suboptions:
      borderPriority:
        description: Border priority associated with a given device. Allowed range for
          Border Priority is 1-9. A lower value indicates higher priority. E.g., a priority
          of 1 takes precedence over 5. Default priority would be set to 10.
        type: str
      borderSessionType:
        description: Border Session Type.
        type: str
        version_added: 4.0.0
      borderWithExternalConnectivity:
        description: Border With External Connectivity (Note True for transit and False
          for non-transit border).
        type: bool
      connectedToInternet:
        description: Connected to Internet.
        type: bool
        version_added: 4.0.0
      deviceManagementIpAddress:
        description: Management Ip Address of the provisioned Device.
        type: str
        version_added: 4.0.0
      deviceRole:
        description: Supported Device Roles in SD-Access fabric. Allowed roles are "Border_Node","Control_Plane_Nod...
          E.g. "Border_Node" or "Border_Node", "Control_Plane_Node" or "Border_Node",
          "Control_Plane_Node","Edge_Node".
        elements: str
        type: list
      externalConnectivityIpPoolName:
        description: External Connectivity IpPool Name.
        type: str
        version_added: 4.0.0
      externalConnectivitySettings:
        description: Sda Fabric Border Device's externalConnectivitySettings.
        elements: dict
        suboptions:
          externalAutonomouSystemNumber:
            description: External Autonomous System Number peer (e.g.,1-65535).
            type: str
            version_added: 4.0.0
          interfaceDescription:
            description: Interface Description.
            type: str
          interfaceName:
            description: Interface Name.
            type: str
            version_added: 4.0.0
          l2Handoff:
            description: Sda Fabric Border Device's l2Handoff.
            elements: dict
            suboptions:
              virtualNetworkName:
                description: Virtual Network Name, that is associated to Fabric Site.
                type: str
                version_added: 4.0.0
              vlanName:
                description: Vlan Name of L2 Handoff.
                type: str
            type: list
          l3Handoff:
            description: Sda Fabric Border Device's l3Handoff.
            elements: dict
            suboptions:
              virtualNetwork:
                description: Sda Fabric Border Device's virtualNetwork.
                suboptions:
                  virtualNetworkName:
                    description: Virtual Network Name, that is associated to Fabric
                      Site.
                    type: str
                    version_added: 4.0.0
                  vlanId:
                    description: Vlan Id (e.g.,2-4096 except for reserved VLANs (1002-1005,
                      2046, 4095)).
                    type: str
                    version_added: 4.0.0
                type: dict
                version_added: 4.0.0
            type: list
            version_added: 4.0.0
        type: list
        version_added: 4.0.0
      externalDomainRoutingProtocolName:
        description: External Domain Routing Protocol Name.
        type: str
      internalAutonomouSystemNumber:
        description: Internal Autonomous System Number.
        type: str
      routeDistributionProtocol:
        description: Route Distribution Protocol for Control Plane Device. Allowed values
          are "LISP_BGP" or "LISP_PUB_SUB". Default value is "LISP_BGP".
        type: str
      sdaTransitNetworkName:
        description: SD-Access Transit Network Name.
        type: str
      siteNameHierarchy:
        description: Site Name Hierarchy of provisioned Device(site should be part of
          Fabric Site).
        type: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA AddBorderDeviceInSDAFabricV1
  description: Complete reference of the AddBorderDeviceInSDAFabricV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-border-device-in-sda-fabric
- name: Cisco DNA Center documentation for SDA DeleteBorderDeviceFromSDAFabricV1
  description: Complete reference of the DeleteBorderDeviceFromSDAFabricV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-border-device-from-sda-fabric
notes:
  - SDK Method used are
    sda.Sda.adds_border_device,
    sda.Sda.deletes_border_device,

  - Paths used are
    post /dna/intent/api/v1/business/sda/border-device,
    delete /dna/intent/api/v1/business/sda/border-device,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.sda_fabric_border_device_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - borderPriority: string
      borderSessionType: string
      borderWithExternalConnectivity: true
      connectedToInternet: true
      deviceManagementIpAddress: string
      deviceRole:
      - string
      externalConnectivityIpPoolName: string
      externalConnectivitySettings:
      - externalAutonomouSystemNumber: string
        interfaceDescription: string
        interfaceName: string
        l2Handoff:
        - virtualNetworkName: string
          vlanName: string
        l3Handoff:
        - virtualNetwork:
            virtualNetworkName: string
            vlanId: string
      externalDomainRoutingProtocolName: string
      internalAutonomouSystemNumber: string
      routeDistributionProtocol: string
      sdaTransitNetworkName: string
      siteNameHierarchy: string

- name: Delete all
  cisco.dnac.sda_fabric_border_device_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    deviceManagementIpAddress: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "status": "string",
      "description": "string",
      "taskId": "string",
      "taskStatusUrl": "string",
      "executionStatusUrl": "string",
      "executionId": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: pan_ha
short_description: Resource module for Pan Ha
description:
- Manage operations create and delete of the resource Pan Ha.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  failedAttempts:
    description: Pan Ha's failedAttempts.
    type: int
  isEnabled:
    description: IsEnabled flag.
    type: bool
  pollingInterval:
    description: Pan Ha's pollingInterval.
    type: int
  primaryHealthCheckNode:
    description: Pan Ha's primaryHealthCheckNode.
    type: str
  secondaryHealthCheckNode:
    description: Pan Ha's secondaryHealthCheckNode.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    sync_ise_node.ReplicationStatus.get_node_replication_status,

  - Paths used are
    get /api/v1/replication-status/{node}
"""

EXAMPLES = r"""
- name: Create
  cisco.ise.pan_ha:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    failedAttempts: 0
    isEnabled: true
    pollingInterval: 0
    primaryHealthCheckNode: string
    secondaryHealthCheckNode: string

- name: Delete all
  cisco.ise.pan_ha:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "isEnabled": true,
        "primaryHealthCheckNode": "string",
        "secondaryHealthCheckNode": "string",
        "pollingInterval": 0,
        "failedAttempts": 0
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: integration_settings_instances_itsm
short_description: Resource module for Integration Settings Instances Itsm
description:
- This module represents an alias of the module integration_settings_instances_itsm_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  data:
    description: Integration Settings Instances Itsm's data.
    suboptions:
      ConnectionSettings:
        description: Integration Settings Instances Itsm's ConnectionSettings.
        suboptions:
          Auth_Password:
            description: Auth Password.
            type: str
          Auth_UserName:
            description: Auth User Name.
            type: str
          Url:
            description: Url.
            type: str
        type: dict
    type: dict
  description:
    description: Description of the setting instance.
    type: str
  dypName:
    description: It can be ServiceNowConnection.
    type: str
  instanceId:
    description: InstanceId path parameter. Instance Id of the Integration setting instance.
    type: str
  name:
    description: Name of the setting instance.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for ITSM Integration CreateITSMIntegrationSettingV1
  description: Complete reference of the CreateITSMIntegrationSettingV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!create-itsm-integration-setting
- name: Cisco DNA Center documentation for ITSM Integration DeleteITSMIntegrationSettingV1
  description: Complete reference of the DeleteITSMIntegrationSettingV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-itsm-integration-setting
- name: Cisco DNA Center documentation for ITSM Integration UpdateITSMIntegrationSettingV1
  description: Complete reference of the UpdateITSMIntegrationSettingV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-itsm-integration-setting
notes:
  - SDK Method used are
    itsm_integration.ItsmIntegration.create_itsm_integration_setting_v1,
    itsm_integration.ItsmIntegration.delete_itsm_integration_setting_v1,
    itsm_integration.ItsmIntegration.update_itsm_integration_setting_v1,

  - Paths used are
    post /dna/intent/api/v1/integration-settings/instances/itsm,
    delete /dna/intent/api/v1/integration-settings/instances/itsm/{instanceId},
    put /dna/intent/api/v1/integration-settings/instances/itsm/{instanceId},
  - It should be noted that this module is an alias of integration_settings_instances_itsm_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.integration_settings_instances_itsm:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    data:
      ConnectionSettings:
        Auth_Password: string
        Auth_UserName: string
        Url: string
    description: string
    dypName: string
    name: string

- name: Update by id
  cisco.dnac.integration_settings_instances_itsm:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    data:
      ConnectionSettings:
        Auth_Password: string
        Auth_UserName: string
        Url: string
    description: string
    dypName: string
    instanceId: string
    name: string

- name: Delete by id
  cisco.dnac.integration_settings_instances_itsm:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    instanceId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "dypId": "string",
      "dypName": "string",
      "name": "string",
      "uniqueKey": "string",
      "dypMajorVersion": 0,
      "description": "string",
      "data": {
        "ConnectionSettings": {
          "Url": "string",
          "Auth_UserName": "string",
          "Auth_Password": "string"
        }
      },
      "createdDate": 0,
      "createdBy": "string",
      "updatedBy": "string",
      "softwareVersionLog": [
        {}
      ],
      "schemaVersion": 0,
      "tenantId": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2018, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: net_put
author: Deepak Agrawal (@dagrawal)
short_description: Copy a file from Ansible Controller to a network device
description:
- This module provides functionality to copy file from Ansible controller to network
  devices.
version_added: 1.0.0
extends_documentation_fragment:
- ansible.netcommon.network_agnostic
options:
  src:
    description:
    - Specifies the source file. The path to the source file can either be the full
      path on the Ansible control host or a relative path from the playbook or role
      root directory.
    required: true
  protocol:
    description:
    - Protocol used to transfer file.
    default: scp
    choices:
    - scp
    - sftp
  dest:
    description:
    - Specifies the destination file. The path to destination file can either be the
      full path or relative path as supported by network_os.
    default:
    - Filename from src and at default directory of user shell on network_os.
    required: false
  mode:
    description:
    - Set the file transfer mode. If mode is set to I(text) then I(src) file will
      go through Jinja2 template engine to replace any vars if present in the src
      file. If mode is set to I(binary) then file will be copied as it is to destination
      device.
    default: binary
    choices:
    - binary
    - text
requirements:
- scp if using protocol=scp with paramiko
notes:
- Some devices need specific configurations to be enabled before scp can work These
  configuration should be pre-configured before using this module e.g ios - C(ip scp
  server enable).
- User privilege to do scp on network device should be pre-configured e.g. ios - need
  user privilege 15 by default for allowing scp.
- Default destination of source file.
"""

EXAMPLES = """
- name: copy file from ansible controller to a network device
  ansible.netcommon.net_put:
    src: running_cfg_ios1.txt

- name: copy file at root dir of flash in slot 3 of sw1(ios)
  ansible.netcommon.net_put:
    src: running_cfg_sw1.txt
    protocol: sftp
    dest: flash3:/running_cfg_sw1.txt
"""

RETURN = """
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: configuration_template_deploy_status_info
short_description: Information module for Configuration Template Deploy Status Info
description:
- This module represents an alias of the module configuration_template_deploy_status_v1_info
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deploymentId:
    description:
    - DeploymentId path parameter. UUID of deployment to retrieve template deployment status.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Configuration Templates StatusOfTemplateDeploymentV1
  description: Complete reference of the StatusOfTemplateDeploymentV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!status-of-template-deployment
notes:
  - SDK Method used are
    configuration_templates.ConfigurationTemplates.get_template_deployment_status,

  - Paths used are
    get /dna/intent/api/v1/template-programmer/template/deploy/status/{deploymentId},
  - It should be noted that this module is an alias of configuration_template_deploy_status_v1_info

"""

EXAMPLES = r"""
- name: Get Configuration Template Deploy Status Info by id
  cisco.dnac.configuration_template_deploy_status_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    deploymentId: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "deploymentId": "string",
      "deploymentName": "string",
      "devices": [
        {
          "detailedStatusMessage": "string",
          "deviceId": "string",
          "duration": "string",
          "endTime": "string",
          "identifier": "string",
          "ipAddress": "string",
          "name": "string",
          "startTime": "string",
          "status": "string",
          "targetType": "string"
        }
      ],
      "duration": "string",
      "endTime": "string",
      "projectName": "string",
      "startTime": "string",
      "status": "string",
      "statusMessage": "string",
      "templateName": "string",
      "templateVersion": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_device_user_defined_field_delete_v1
short_description: Resource module for Network Device User Defined Field Delete V1
description:
- Manage operation delete of the resource Network Device User Defined Field Delete V1.
- >
   Remove a User-Defined-Field from device. Name of UDF has to be passed as the query parameter. Please note that
   Global UDF will not be deleted by this operation.
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  deviceId:
    description: DeviceId path parameter. UUID of device from which UDF has to be removed.
    type: str
  name:
    description: Name query parameter. Name of UDF to be removed.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices RemoveUserDefinedFieldFromDeviceV1
  description: Complete reference of the RemoveUserDefinedFieldFromDeviceV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!remove-user-defined-field-from-device
notes:
  - SDK Method used are
    devices.Devices.remove_user_defined_field_from_device_v1,

  - Paths used are
    delete /dna/intent/api/v1/network-device/{deviceId}/user-defined-field,

"""

EXAMPLES = r"""
- name: Delete all
  cisco.dnac.network_device_user_defined_field_delete_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    deviceId: string
    name: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: cloudtrail_info
version_added: 5.0.0
short_description: Gather information about trails in AWS Cloud Trail
description:
  - Gather information about trails in AWS CloudTrail.
author: "Gomathi Selvi Srinivasan (@GomathiselviS)"
options:
  trail_names:
    type: list
    elements: str
    default: []
    description:
      - Specifies a list of trail names, trail ARNs, or both, of the trails to describe.
      - If an empty list is specified, information for the trail in the current region is returned.
  include_shadow_trails:
    type: bool
    default: true
    description: Specifies whether to include shadow trails in the response.
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Gather information about all trails
- amazon.aws.cloudtrail_info:

# Gather information about a particular trail
- amazon.aws.cloudtrail_info:
    trail_names:
      - arn:aws:cloudtrail:us-east-2:123456789012:trail/MyTrail
"""

RETURN = r"""
trail_list:
    description: List of trail objects. Each element consists of a dict with all the information related to that cloudtrail.
    type: list
    elements: dict
    returned: always
    contains:
        name:
            description: Name of the trail.
            type: str
            sample: "MyTrail"
        s3_bucket_name:
            description: Name of the Amazon S3 bucket into which CloudTrail delivers the trail files.
            type: str
            sample: "aws-cloudtrail-logs-xxxx"
        s3_key_prefix:
            description: Amazon S3 key prefix that comes after the name of the bucket that is designated for log file delivery.
            type: str
            sample: "xxxx"
        sns_topic_arn:
            description: ARN of the Amazon SNS topic that CloudTrail uses to send notifications when log files are delivered.
            type: str
            sample: "arn:aws:sns:us-east-2:123456789012:MyTopic"
        include_global_service_events:
            description: If True, AWS API calls from AWS global services such as IAM are included.
            type: bool
            sample: true
        is_multi_region_trail:
            description: Specifies whether the trail exists only in one region or exists in all regions.
            type: bool
            sample: true
        home_region:
            description: The region in which the trail was created.
            type: str
            sample: "us-east-1"
        trail_arn:
            description: Specifies the ARN of the trail.
            type: str
            sample: "arn:aws:cloudtrail:us-east-2:123456789012:trail/MyTrail"
        log_file_validation_enabled:
            description: Specifies whether log file validation is enabled.
            type: bool
            sample: true
        cloud_watch_logs_log_group_arn:
            description: Specifies an ARN, that represents the log group to which CloudTrail logs will be delivered.
            type: str
            sample: "arn:aws:sns:us-east-2:123456789012:Mylog"
        cloud_watch_logs_role_arn:
            description: Specifies the role for the CloudWatch Logs endpoint to assume to write to a user's log group.
            type: str
            sample: "arn:aws:sns:us-east-2:123456789012:Mylog"
        kms_key_id:
            description: Specifies the KMS key ID that encrypts the logs delivered by CloudTrail.
            type: str
            sample: "arn:aws:kms:us-east-2:123456789012:key/12345678-1234-1234-1234-123456789012"
        has_custom_event_selectors:
            description: Specifies if the trail has custom event selectors.
            type: bool
            sample: true
        has_insight_selectors:
            description: Specifies whether a trail has insight types specified in an InsightSelector list.
            type: bool
            sample: true
        is_organization_trail:
            description: Specifies whether the trail is an organization trail.
            type: bool
            sample: true
        is_logging:
            description: Whether the CloudTrail is currently logging AWS API calls.
            type: bool
            sample: true
        latest_delivery_error:
            description: Displays any Amazon S3 error that CloudTrail encountered when attempting to deliver log files to the designated bucket.
            type: str
        latest_notification_attempt_time:
            description: Specifies the date and time that CloudTrail last attempt to deliver a notification.
            type: str
        latest_notification_attempt_succeeded:
            description: Specifies the date and time that CloudTrail last successful attempt to deliver a notification.
            type: str
        latest_notification_error:
            description: Displays any Amazon SNS error that CloudTrail encountered when attempting to send a notification.
            type: str
        latest_delivery_attempt_succeeded:
            description: Specifies the date and time that CloudTrail last successful attempt to deliver log files to an account's Amazon S3 bucket.
            type: str
        latest_delivery_attempt_time:
            description: Specifies the date and time that CloudTrail last attempt to deliver log files to an account's Amazon S3 bucket.
            type: str
        latest_delivery_time:
            description: Specifies the date and time that CloudTrail last delivered log files to an account's Amazon S3 bucket.
            type: str
        start_logging_time:
            description: Specifies the most recent date and time when CloudTrail started recording API calls for an AWS account.
            type: str
        stop_logging_time:
            description: Specifies the most recent date and time when CloudTrail stopped recording API calls for an AWS account.
            type: str
        latest_cloud_watch_logs_delivery_error:
            description: Displays any CloudWatch Logs error that CloudTrail encountered when attempting to deliver logs to CloudWatch Logs.
            type: str
        latest_cloud_watch_logs_delivery_time:
            description: Displays the most recent date and time when CloudTrail delivered logs to CloudWatch Logs.
            type: str
        latest_digest_delivery_time:
            description: Specifies the date and time that CloudTrail last delivered a digest file to an account's Amazon S3 bucket.
            type: str
        latest_digest_delivery_error:
            description: Displays any Amazon S3 error that CloudTrail encountered when attempting to deliver a digest file to the designated bucket.
            type: str
        resource_id:
            description: Specifies the ARN of the resource.
            type: str
        tags:
            description: Any tags assigned to the cloudtrail.
            type: dict
            returned: always
            sample: "{ 'my_tag_key': 'my_tag_value' }"
"""

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict


def get_trails(connection, module):
    all_trails = []
    try:
        result = connection.get_paginator("list_trails")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to get the trails.")
    for trail in result.paginate():
        all_trails.extend(list_cloud_trails(trail))
    return all_trails


def list_cloud_trails(trail_dict):
    return [x["TrailARN"] for x in trail_dict["Trails"]]


def get_trail_detail(connection, module):
    output = {}
    trail_name_list = module.params.get("trail_names")
    include_shadow_trails = module.params.get("include_shadow_trails")
    if not trail_name_list:
        trail_name_list = get_trails(connection, module)
    try:
        result = connection.describe_trails(
            trailNameList=trail_name_list, includeShadowTrails=include_shadow_trails, aws_retry=True
        )
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to get the trails.")
    # Turn the boto3 result in to ansible_friendly_snaked_names
    snaked_cloud_trail = []
    for cloud_trail in result["trailList"]:
        try:
            status_dict = connection.get_trail_status(Name=cloud_trail["TrailARN"], aws_retry=True)
            cloud_trail.update(status_dict)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Failed to get the trail status")
        try:
            tag_list = connection.list_tags(ResourceIdList=[cloud_trail["TrailARN"]])
            for tag_dict in tag_list["ResourceTagList"]:
                cloud_trail.update(tag_dict)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.warn(f"Failed to get the trail tags - {e}")
        snaked_cloud_trail.append(camel_dict_to_snake_dict(cloud_trail))

    # Turn the boto3 result in to ansible friendly tag dictionary
    for tr in snaked_cloud_trail:
        if "tags_list" in tr:
            tr["tags"] = boto3_tag_list_to_ansible_dict(tr["tags_list"], "key", "value")
            del tr["tags_list"]
        if "response_metadata" in tr:
            del tr["response_metadata"]
    output["trail_list"] = snaked_cloud_trail
    return output


def main():
    argument_spec = dict(
        trail_names=dict(type="list", elements="str", default=[]),
        include_shadow_trails=dict(type="bool", default=True),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    try:
        connection = module.client("cloudtrail", retry_decorator=AWSRetry.jittered_backoff())
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")
    result = get_trail_detail(connection, module)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_local_user_policy
short_description: Local User Policy configuration for Cisco Intersight
description:
  - Local User Policy configuration for Cisco Intersight.
  - Used to configure local users on endpoint devices.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles and Policies that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Local User Policy.
      - The name must be between 1 and 62 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  description:
    description:
      - The user-defined description of the Local User policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  enforce_strong_password:
    description:
      - If true, enables a strong password policy.
      - Strong password requirements:.
      - A. The password must have a minimum of 8 and a maximum of 20 characters.
      - B. The password must not contain the User's Name.
      - C. The password must contain characters from three of the following four categories.
      - 1) English uppercase characters (A through Z).
      - 2) English lowercase characters (a through z).
      - 3) Base 10 digits (0 through 9).
      - 4) Non-alphabetic characters (! , @, '#', $, %, ^, &, *, -, _, +, =).
    type: bool
    default: true
  enable_password_expiry:
    description:
      - Enables password expiry on the endpoint.
    type: bool
    default: false
  password_history:
    description:
      - Specifies number of times a password cannot repeat when changed (value between 0 and 5).
      - Entering 0 disables this option.
    type: int
    default: 5
  local_users:
    description:
      - List of local users on the endpoint.
      - An admin user already exists on the endpoint.
      - Add the admin user here only if you want to change the password, or enable or disable the user.
      - To add admin user, provide a username as 'admin', select the admin user role, and then proceed.
    type: list
    elements: dict
    suboptions:
      username:
        description:
          - Name of the user created on the endpoint.
        type: str
        required: true
      enable:
        description:
          - Enable or disable the user.
        type: bool
        default: true
      role:
        description:
          - Roles associated with the user on the endpoint.
        type: str
        choices: [admin, readonly, user]
        required: true
      password:
        description:
          - Valid login password of the user.
        type: str
        required: true
  purge:
    description:
      - The purge argument instructs the module to consider the resource definition absolute.
      - If true, any previously configured usernames will be removed from the policy with the exception of the `admin` user which cannot be deleted.
    type: bool
    default: false
  always_update_password:
    description:
      - Since passwords are not returned by the API and are encrypted on the endpoint, this option will instruct the module when to change the password.
      - If true, the password for each user will always be updated in the policy.
      - If false, the password will be updated only if the user is created.
    type: bool
    default: false
author:
  - David Soper (@dsoper2)
'''

EXAMPLES = r'''
- name: Configure Local User policy
  intersight_local_user_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: guest-admin
    tags:
      - Key: username
        Value: guest
    description: User named guest with admin role
    local_users:
      - username: guest
        role: admin
        password: vault_guest_password
      - username: reader
        role: readonly
        password: vault_reader_password

- name: Delete Local User policy
  intersight_local_user_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: guest-admin
    state: absent
'''

RETURN = r'''
api_repsonse:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Description": "User named guest with admin role",
        "EndPointUserRoles": [
            {
                "ChangePassword": true,
                "Enabled": true
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec, compare_values


def main():
    local_user = dict(
        username=dict(type='str', required=True),
        enable=dict(type='bool', default=True),
        role=dict(type='str', choices=['admin', 'readonly', 'user'], required=True),
        password=dict(type='str', required=True, no_log=True),
    )
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        enforce_strong_password=dict(type='bool', default=True, no_log=False),
        enable_password_expiry=dict(type='bool', default=False, no_log=False),
        password_history=dict(type='int', default=5, no_log=False),
        local_users=dict(type='list', elements='dict', options=local_user),
        purge=dict(type='bool', default=False),
        always_update_password=dict(type='bool', default=False, no_log=False),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    organization_moid = None
    # get Organization Moid
    intersight.get_resource(
        resource_path='/organization/Organizations',
        query_params={
            '$filter': "Name eq '" + intersight.module.params['organization'] + "'",
            '$select': 'Moid',
        },
    )
    if intersight.result['api_response'].get('Moid'):
        # resource exists and moid was returned
        organization_moid = intersight.result['api_response']['Moid']
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # get the current state of the resource
    filter_str = "Name eq '" + intersight.module.params['name'] + "'"
    filter_str += "and Organization.Moid eq '" + organization_moid + "'"
    intersight.get_resource(
        resource_path='/iam/EndPointUserPolicies',
        query_params={
            '$filter': filter_str,
            '$expand': 'EndPointUserRoles($expand=EndPointRole,EndPointUser),Organization',
        },
    )

    user_policy_moid = None
    resource_values_match = False
    if intersight.result['api_response'].get('Moid'):
        # resource exists and moid was returned
        user_policy_moid = intersight.result['api_response']['Moid']
        #
        # always_update_password
        #   false: compare expected vs. actual (won't check passwords)
        #   true: no compare
        #
        if module.params['state'] == 'present' and not module.params['always_update_password']:
            # Create api body used to check current state
            end_point_user_roles = []
            for user in intersight.module.params['local_users']:
                end_point_user_roles.append(
                    {
                        'Enabled': user['enable'],
                        'EndPointRole': [
                            {
                                'Name': user['role'],
                                'Type': 'IMC',
                            },
                        ],
                        'EndPointUser': {
                            'Name': user['username'],
                        },
                    }
                )
            intersight.api_body = {
                'Name': intersight.module.params['name'],
                'Tags': intersight.module.params['tags'],
                'Description': intersight.module.params['description'],
                'PasswordProperties': {
                    'EnforceStrongPassword': intersight.module.params['enforce_strong_password'],
                    'EnablePasswordExpiry': intersight.module.params['enable_password_expiry'],
                    'PasswordHistory': intersight.module.params['password_history'],
                },
                'EndPointUserRoles': end_point_user_roles,
                'Organization': {
                    'Name': intersight.module.params['organization'],
                },
            }
            resource_values_match = compare_values(intersight.api_body, intersight.result['api_response'])
        elif module.params['state'] == 'absent':
            intersight.delete_resource(
                moid=user_policy_moid,
                resource_path='/iam/EndPointUserPolicies',
            )
            user_policy_moid = None

    if module.params['state'] == 'present' and not resource_values_match:
        intersight.api_body = {
            'Name': intersight.module.params['name'],
            'Tags': intersight.module.params['tags'],
            'Description': intersight.module.params['description'],
            'PasswordProperties': {
                'EnforceStrongPassword': intersight.module.params['enforce_strong_password'],
                'EnablePasswordExpiry': intersight.module.params['enable_password_expiry'],
                'PasswordHistory': intersight.module.params['password_history'],
            },
            'Organization': {
                'Moid': organization_moid
            },
        }

        if module.params['purge']:
            # update existing resource and purge any existing users
            if intersight.result['api_response'].get('EndPointUserRoles'):
                for end_point_user_role in intersight.result['api_response']['EndPointUserRoles']:
                    intersight.delete_resource(
                        moid=end_point_user_role['Moid'],
                        resource_path='/iam/EndPointUserRoles',
                    )
        # configure the top-level policy resource
        intersight.result['api_response'] = {}
        intersight.configure_resource(
            moid=user_policy_moid,
            resource_path='/iam/EndPointUserPolicies',
            body=intersight.api_body,
            query_params={
                '$filter': "Name eq '" + intersight.module.params['name'] + "'",
            },
        )
        if intersight.result['api_response'].get('Moid'):
            # resource exists and moid was returned
            user_policy_moid = intersight.result['api_response']['Moid']

        # EndPointUser local_users list config
        for user in intersight.module.params['local_users']:
            # check for existing user in this organization
            filter_str = "Name eq '" + user['username'] + "'"
            filter_str += "and Organization.Moid eq '" + organization_moid + "'"
            intersight.get_resource(
                resource_path='/iam/EndPointUsers',
                query_params={
                    '$filter': filter_str
                },
            )
            user_moid = None
            if intersight.result['api_response'].get('Moid'):
                # resource exists and moid was returned
                user_moid = intersight.result['api_response']['Moid']
            else:
                # create user if it doesn't exist
                intersight.api_body = {
                    'Name': user['username'],
                }
                if organization_moid:
                    intersight.api_body['Organization'] = {
                        'Moid': organization_moid,
                    }
                intersight.configure_resource(
                    moid=None,
                    resource_path='/iam/EndPointUsers',
                    body=intersight.api_body,
                    query_params={
                        '$filter': "Name eq '" + user['username'] + "'",
                    },
                )
                user_moid = None
                if intersight.result['api_response'].get('Moid'):
                    # resource exists and moid was returned
                    user_moid = intersight.result['api_response']['Moid']
            # GET EndPointRole Moid
            intersight.get_resource(
                resource_path='/iam/EndPointRoles',
                query_params={
                    '$filter': "Name eq '" + user['role'] + "' and Type eq 'IMC'",
                },
            )
            end_point_role_moid = None
            if intersight.result['api_response'].get('Moid'):
                # resource exists and moid was returned
                end_point_role_moid = intersight.result['api_response']['Moid']
            # EndPointUserRole config
            intersight.api_body = {
                'EndPointUser': {
                    'Moid': user_moid,
                },
                'EndPointRole': [
                    {
                        'Moid': end_point_role_moid,
                    }
                ],
                'Password': user['password'],
                'Enabled': user['enable'],
                'EndPointUserPolicy': {
                    'Moid': user_policy_moid,
                },
            }
            intersight.configure_resource(
                moid=None,
                resource_path='/iam/EndPointUserRoles',
                body=intersight.api_body,
                query_params={
                    '$filter': "EndPointUserPolicy.Moid eq '" + user_policy_moid + "'",
                },
            )

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: endpoints
short_description: Resource module for Endpoints
description:
- Manage operations create, update and delete of the resource Endpoints.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  connectedLinks:
    description: Endpoints's connectedLinks.
    type: dict
  customAttributes:
    description: Endpoints's customAttributes.
    type: dict
  description:
    description: Endpoints's description.
    type: str
  deviceType:
    description: Endpoints's deviceType.
    type: str
  groupId:
    description: Endpoints's groupId.
    type: str
  hardwareRevision:
    description: Endpoints's hardwareRevision.
    type: str
  id:
    description: Endpoints's id.
    type: str
  identityStore:
    description: Endpoints's identityStore.
    type: str
  identityStoreId:
    description: Endpoints's identityStoreId.
    type: str
  ipAddress:
    description: Endpoints's ipAddress.
    type: str
  mac:
    description: Endpoints's mac.
    type: str
  mdmAttributes:
    description: Endpoints's mdmAttributes.
    type: dict
  name:
    description: Endpoints's name.
    type: str
  portalUser:
    description: Endpoints's portalUser.
    type: str
  productId:
    description: Endpoints's productId.
    type: str
  profileId:
    description: Endpoints's profileId.
    type: str
  protocol:
    description: Endpoints's protocol.
    type: str
  serialNumber:
    description: Endpoints's serialNumber.
    type: str
  softwareRevision:
    description: Endpoints's softwareRevision.
    type: str
  staticGroupAssignment:
    description: StaticGroupAssignment flag.
    type: bool
  staticProfileAssignment:
    description: StaticProfileAssignment flag.
    type: bool
  value:
    description: Value path parameter. The id or MAC of the endpoint.
    type: str
  vendor:
    description: Endpoints's vendor.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    endpoints.Endpoints.create_end_point,
    endpoints.Endpoints.delete_endpoint,
    endpoints.Endpoints.update_endpoint,

  - Paths used are
    post /api/v1/endpoint,
    delete /api/v1/endpoint/{value},
    put /api/v1/endpoint/{value},

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.endpoints:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    connectedLinks: {}
    customAttributes: {}
    description: string
    deviceType: string
    groupId: string
    hardwareRevision: string
    id: string
    identityStore: string
    identityStoreId: string
    ipAddress: string
    mac: string
    mdmAttributes: {}
    name: string
    portalUser: string
    productId: string
    profileId: string
    protocol: string
    serialNumber: string
    softwareRevision: string
    staticGroupAssignment: true
    staticProfileAssignment: true
    vendor: string

- name: Update by id
  cisco.ise.endpoints:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    connectedLinks: {}
    customAttributes: {}
    description: string
    deviceType: string
    groupId: string
    hardwareRevision: string
    id: string
    identityStore: string
    identityStoreId: string
    ipAddress: string
    mac: string
    mdmAttributes: {}
    name: string
    portalUser: string
    productId: string
    profileId: string
    protocol: string
    serialNumber: string
    softwareRevision: string
    staticGroupAssignment: true
    staticProfileAssignment: true
    value: string
    vendor: string

- name: Delete by id
  cisco.ise.endpoints:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    value: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "connectedLinks": {},
      "customAttributes": {},
      "description": "string",
      "deviceType": "string",
      "groupId": "string",
      "hardwareRevision": "string",
      "id": "string",
      "identityStore": "string",
      "identityStoreId": "string",
      "ipAddress": "string",
      "mac": "string",
      "mdmAttributes": {},
      "name": "string",
      "portalUser": "string",
      "productId": "string",
      "profileId": "string",
      "protocol": "string",
      "serialNumber": "string",
      "softwareRevision": "string",
      "staticGroupAssignment": true,
      "staticProfileAssignment": true,
      "vendor": "string"
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "connectedLinks": {},
      "customAttributes": {},
      "description": "string",
      "deviceType": "string",
      "groupId": "string",
      "hardwareRevision": "string",
      "id": "string",
      "identityStore": "string",
      "identityStoreId": "string",
      "ipAddress": "string",
      "mac": "string",
      "mdmAttributes": {},
      "name": "string",
      "portalUser": "string",
      "productId": "string",
      "profileId": "string",
      "protocol": "string",
      "serialNumber": "string",
      "softwareRevision": "string",
      "staticGroupAssignment": true,
      "staticProfileAssignment": true,
      "vendor": "string"
    }
"""



from __future__ import absolute_import, division, print_function

__metaclass__ = type

import pytest

from awx.main.models import Organization
from awx.main.models.oauth import OAuth2Application


@pytest.mark.django_db
def test_create_application(run_module, admin_user):
    org = Organization.objects.create(name='foo')

    module_args = {
        'name': 'foo_app',
        'description': 'barfoo',
        'state': 'present',
        'authorization_grant_type': 'password',
        'client_type': 'public',
        'organization': 'foo',
    }

    result = run_module('application', module_args, admin_user)
    assert result.get('changed'), result

    application = OAuth2Application.objects.get(name='foo_app')
    assert application.description == 'barfoo'
    assert application.organization_id == org.id



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: certificate_profile
short_description: Resource module for Certificate Profile
description:
- Manage operations create and update of the resource Certificate Profile.
- This API allows the client to create a certificate profile.
- This API allows the client to update a certificate profile.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  allowedAsUserName:
    description: AllowedAsUserName flag.
    type: bool
  certificateAttributeName:
    description: Attribute name of the Certificate Profile - used only when CERTIFICATE
      is chosen in usernameFrom. Allowed values - SUBJECT_COMMON_NAME - SUBJECT_ALTERNATIVE_NAME
      - SUBJECT_SERIAL_NUMBER - SUBJECT - SUBJECT_ALTERNATIVE_NAME_OTHER_NAME - SUBJECT_ALTERNATIVE_NAME_EMAIL
      - SUBJECT_ALTERNATIVE_NAME_DNS. - Additional internal value ALL_SUBJECT_AND_ALTERNATIVE_NAMES
      is used automatically when usernameFrom=UPN.
    type: str
  description:
    description: Certificate Profile's description.
    type: str
  externalIdentityStoreName:
    description: Referred IDStore name for the Certificate Profile or not applicable
      in case no identity store is chosen.
    type: str
  id:
    description: Certificate Profile's id.
    type: str
  matchMode:
    description: Match mode of the Certificate Profile. Allowed values - NEVER - RESOLVE_IDENTITY_AMBIGUITY
      - BINARY_COMPARISON.
    type: str
  name:
    description: Certificate Profile's name.
    type: str
  usernameFrom:
    description: The attribute in the certificate where the user name should be taken
      from. Allowed values - CERTIFICATE (for a specific attribute as defined in certificateAttributeName)
      - UPN (for using any Subject or Alternative Name Attributes in the Certificate
      - an option only in AD).
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    certificate_profile.CertificateProfile.create_certificate_profile,
    certificate_profile.CertificateProfile.update_certificate_profile_by_id,

  - Paths used are
    post /ers/config/certificateprofile,
    put /ers/config/certificateprofile/{id},

"""

EXAMPLES = r"""
- name: Update by id
  cisco.ise.certificate_profile:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    allowedAsUserName: true
    certificateAttributeName: string
    description: string
    externalIdentityStoreName: string
    id: string
    matchMode: string
    name: string
    usernameFrom: string

- name: Create
  cisco.ise.certificate_profile:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    allowedAsUserName: true
    certificateAttributeName: string
    description: string
    externalIdentityStoreName: string
    id: string
    matchMode: string
    name: string
    usernameFrom: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "name": "string",
      "description": "string",
      "externalIdentityStoreName": "string",
      "certificateAttributeName": "string",
      "allowedAsUserName": true,
      "matchMode": "string",
      "usernameFrom": "string",
      "link": {
        "rel": "string",
        "href": "string",
        "type": "string"
      }
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "UpdatedFieldsList": {
        "updatedField": [
          {
            "field": "string",
            "oldValue": "string",
            "newValue": "string"
          }
        ]
      }
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2020 GuopengLin, (@t-glin)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_vpnsitelink_info
version_added: '1.5.0'
short_description: Get VpnSiteLink info
description:
    - Get info of Vpn Site Link relate infomation.
options:
    resource_group:
        description:
            - The resource group name of the VpnSite.
        required: true
        type: str
    vpn_site_name:
        description:
            - The name of the Vpn Site.
        required: true
        type: str
    name:
        description:
            - The name of the VpnSiteLink being retrieved.
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
author:
    - Fred-Sun (@Fred-Sun)
    - Haiyuan Zhang (@haiyuazhang)

'''

EXAMPLES = '''
- name: Get Vpn Site Link info by the name
  azure_rm_vpnsitelink_info:
    resource_group: myResourceGroup
    name: vpnSiteLink1
    vpn_site_name: vpnSite1


- name: Get Vpn Site Links by the Vpn Site
  azure_rm_vpnsitelink_info:
    resource_group: myResourceGroup
    vpn_site_name: vpnSite1
'''

RETURN = '''
vpn_site_links:
    description:
        - A list of dict results where the key is the name of the VpnSiteLink and the values are the facts for that VpnSiteLink.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: /subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/vpnSites/fred/vpnSiteLinks/azureuser
        etag:
            description:
                - A unique read-only string that changes whenever the resource is updated.
            returned: always
            type: str
            sample: 1ec5c61b-d66f-4b1c-b7b5-f27d0a9ad9d3
        name:
            description:
                - The name of the resource that is unique within a resource group.
                - This name can be used to access the resource.
            returned: always
            type: str
            sample: azureuser
        type:
            description:
                - Resource type.
            returned: always
            type: str
            sample: Microsoft.Network/vpnSites/vpnSiteLinks
        link_properties:
            description:
                - The link provider properties.
            returned: always
            type: complex
            contains:
                link_provider_name:
                    description:
                        - Name of the link provider.
                    returned: always
                    type: str
                    sample: azureuser
                link_speed_in_mbps:
                    description:
                        - Link speed.
                    returned: always
                    type: int
                    sample: 100
        ip_address:
            description:
                - The ip-address for the vpn-site-link.
            returned: always
            type: str
            sample: 192.168.33.223
        provisioning_state:
            description:
                - The provisioning state of the VPN site link resource.
            returned: always
            type: str
            sample: Succeeded
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBase
try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMVpnSiteLinkInfo(AzureRMModuleBase):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            vpn_site_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str'
            )
        )

        self.resource_group = None
        self.vpn_site_name = None
        name = None

        self.results = dict(changed=False)
        self.state = None

        super(AzureRMVpnSiteLinkInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=True)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if (self.resource_group is not None and self.vpn_site_name is not None and self.name is not None):
            self.results['vpn_site_links'] = self.format_item(self.get())
        elif (self.resource_group is not None and self.vpn_site_name is not None):
            self.results['vpn_site_links'] = self.format_item(self.list_by_vpn_site())
        return self.results

    def get(self):
        response = None

        try:
            response = self.network_client.vpn_site_links.get(resource_group_name=self.resource_group,
                                                              vpn_site_name=self.vpn_site_name,
                                                              vpn_site_link_name=self.name)
        except ResourceNotFoundError as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')

        return response

    def list_by_vpn_site(self):
        response = None

        try:
            response = self.network_client.vpn_site_links.list_by_vpn_site(resource_group_name=self.resource_group,
                                                                           vpn_site_name=self.vpn_site_name)
        except ResourceNotFoundError as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')

        return response

    def format_item(self, item):
        if hasattr(item, 'as_dict'):
            return [item.as_dict()]
        elif item is not None:
            result = []
            items = list(item)
            for tmp in items:
                result.append(tmp.as_dict())
            return result
        else:
            return None


def main():
    AzureRMVpnSiteLinkInfo()


if __name__ == '__main__':
    main()



#
# Copyright 2018 Red Hat Inc.
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type

from ansible_collections.ansible.netcommon.plugins.action.network import (
    ActionModule as ActionNetworkModule,
)


class ActionModule(ActionNetworkModule):
    def run(self, tmp=None, task_vars=None):
        del tmp  # tmp no longer has any effect

        self._config_module = True
        if self._play_context.connection.split(".")[-1] != "network_cli":
            return {
                "failed": True,
                "msg": "Connection type %s is not valid for cli_config module"
                % self._play_context.connection,
            }

        return super(ActionModule, self).run(task_vars=task_vars)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: wireless_profile_info
short_description: Information module for Wireless Profile Info
description:
- This module represents an alias of the module wireless_profile_v1_info
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  profileName:
    description:
    - ProfileName query parameter. Wireless Network Profile Name.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Wireless GetWirelessProfileV1
  description: Complete reference of the GetWirelessProfileV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-wireless-profile
notes:
  - SDK Method used are
    wireless.Wireless.get_wireless_profile_v1,

  - Paths used are
    get /dna/intent/api/v1/wireless/profile,
  - It should be noted that this module is an alias of wireless_profile_v1_info

"""

EXAMPLES = r"""
- name: Get all Wireless Profile Info
  cisco.dnac.wireless_profile_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    profileName: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "profileDetails": {
          "name": "string",
          "sites": [
            "string"
          ],
          "ssidDetails": [
            {
              "name": "string",
              "type": "string",
              "enableFabric": true,
              "flexConnect": {
                "enableFlexConnect": true,
                "localToVlan": 0
              },
              "interfaceName": "string"
            }
          ]
        }
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: configuration_template_v1_info
short_description: Information module for Configuration Template V1
description:
- Get all Configuration Template V1.
- Get Configuration Template V1 by id.
- Details of the template by its id.
- List the templates available.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  projectId:
    description:
    - ProjectId query parameter. Filter template(s) based on project UUID.
    type: str
  softwareType:
    description:
    - SoftwareType query parameter. Filter template(s) based software type.
    type: str
  softwareVersion:
    description:
    - SoftwareVersion query parameter. Filter template(s) based softwareVersion.
    type: str
  productFamily:
    description:
    - ProductFamily query parameter. Filter template(s) based on device family.
    type: str
  productSeries:
    description:
    - ProductSeries query parameter. Filter template(s) based on device series.
    type: str
  productType:
    description:
    - ProductType query parameter. Filter template(s) based on device type.
    type: str
  filterConflictingTemplates:
    description:
    - FilterConflictingTemplates query parameter. Filter template(s) based on confliting templates.
    type: bool
  tags:
    description:
    - Tags query parameter. Filter template(s) based on tags.
    elements: str
    type: list
  projectNames:
    description:
    - ProjectNames query parameter. Filter template(s) based on project names.
    elements: str
    type: list
  unCommitted:
    description:
    - UnCommitted query parameter. Filter template(s) based on template commited or not.
    type: bool
  sortOrder:
    description:
    - SortOrder query parameter. Sort Order Ascending (asc) or Descending (des).
    type: str
  templateId:
    description:
    - TemplateId path parameter. TemplateId(UUID) to get details of the template.
    type: str
  latestVersion:
    description:
    - LatestVersion query parameter. LatestVersion flag to get the latest versioned template.
    type: bool
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Configuration Templates GetsDetailsOfAGivenTemplateV1
  description: Complete reference of the GetsDetailsOfAGivenTemplateV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!gets-details-of-a-given-template
- name: Cisco DNA Center documentation for Configuration Templates GetsTheTemplatesAvailableV1
  description: Complete reference of the GetsTheTemplatesAvailableV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!gets-the-templates-available
notes:
  - SDK Method used are
    configuration_templates.ConfigurationTemplates.get_template_details,
    configuration_templates.ConfigurationTemplates.gets_the_templates_available_v1,

  - Paths used are
    get /dna/intent/api/v1/template-programmer/template,
    get /dna/intent/api/v1/template-programmer/template/{templateId},

"""

EXAMPLES = r"""
- name: Get all Configuration Template V1
  cisco.dnac.configuration_template_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    projectId: string
    softwareType: string
    softwareVersion: string
    productFamily: string
    productSeries: string
    productType: string
    filterConflictingTemplates: True
    tags: []
    projectNames: []
    unCommitted: True
    sortOrder: string
  register: result

- name: Get Configuration Template V1 by id
  cisco.dnac.configuration_template_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    latestVersion: True
    templateId: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "tags": [
        {
          "id": "string",
          "name": "string"
        }
      ],
      "author": "string",
      "composite": true,
      "containingTemplates": [
        {
          "tags": [
            {
              "id": "string",
              "name": "string"
            }
          ],
          "composite": true,
          "description": "string",
          "deviceTypes": [
            {
              "productFamily": "string",
              "productSeries": "string",
              "productType": "string"
            }
          ],
          "id": "string",
          "language": "string",
          "name": "string",
          "projectName": "string",
          "rollbackTemplateParams": [
            {
              "binding": "string",
              "customOrder": 0,
              "dataType": "string",
              "defaultValue": "string",
              "description": "string",
              "displayName": "string",
              "group": "string",
              "id": "string",
              "instructionText": "string",
              "key": "string",
              "notParam": true,
              "order": 0,
              "paramArray": true,
              "parameterName": "string",
              "provider": "string",
              "range": [
                {
                  "id": "string",
                  "maxValue": 0,
                  "minValue": 0
                }
              ],
              "required": true,
              "selection": {
                "defaultSelectedValues": [
                  "string"
                ],
                "id": "string",
                "selectionType": "string",
                "selectionValues": {}
              }
            }
          ],
          "templateContent": "string",
          "templateParams": [
            {
              "binding": "string",
              "customOrder": 0,
              "dataType": "string",
              "defaultValue": "string",
              "description": "string",
              "displayName": "string",
              "group": "string",
              "id": "string",
              "instructionText": "string",
              "key": "string",
              "notParam": true,
              "order": 0,
              "paramArray": true,
              "parameterName": "string",
              "provider": "string",
              "range": [
                {
                  "id": "string",
                  "maxValue": 0,
                  "minValue": 0
                }
              ],
              "required": true,
              "selection": {
                "defaultSelectedValues": [
                  "string"
                ],
                "id": "string",
                "selectionType": "string",
                "selectionValues": {}
              }
            }
          ],
          "version": "string"
        }
      ],
      "createTime": 0,
      "customParamsOrder": true,
      "description": "string",
      "deviceTypes": [
        {
          "productFamily": "string",
          "productSeries": "string",
          "productType": "string"
        }
      ],
      "failurePolicy": "string",
      "id": "string",
      "language": "string",
      "lastUpdateTime": 0,
      "latestVersionTime": 0,
      "name": "string",
      "parentTemplateId": "string",
      "projectId": "string",
      "projectName": "string",
      "rollbackTemplateContent": "string",
      "rollbackTemplateParams": [
        {
          "binding": "string",
          "customOrder": 0,
          "dataType": "string",
          "defaultValue": "string",
          "description": "string",
          "displayName": "string",
          "group": "string",
          "id": "string",
          "instructionText": "string",
          "key": "string",
          "notParam": true,
          "order": 0,
          "paramArray": true,
          "parameterName": "string",
          "provider": "string",
          "range": [
            {
              "id": "string",
              "maxValue": 0,
              "minValue": 0
            }
          ],
          "required": true,
          "selection": {
            "defaultSelectedValues": [
              "string"
            ],
            "id": "string",
            "selectionType": "string",
            "selectionValues": {}
          }
        }
      ],
      "softwareType": "string",
      "softwareVariant": "string",
      "softwareVersion": "string",
      "templateContent": "string",
      "templateParams": [
        {
          "binding": "string",
          "customOrder": 0,
          "dataType": "string",
          "defaultValue": "string",
          "description": "string",
          "displayName": "string",
          "group": "string",
          "id": "string",
          "instructionText": "string",
          "key": "string",
          "notParam": true,
          "order": 0,
          "paramArray": true,
          "parameterName": "string",
          "provider": "string",
          "range": [
            {
              "id": "string",
              "maxValue": 0,
              "minValue": 0
            }
          ],
          "required": true,
          "selection": {
            "defaultSelectedValues": [
              "string"
            ],
            "id": "string",
            "selectionType": "string",
            "selectionValues": {}
          }
        }
      ],
      "validationErrors": {
        "rollbackTemplateErrors": {},
        "templateErrors": {},
        "templateId": "string",
        "templateVersion": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_check_threat_ioc_feed
short_description: Check if a target can reach or parse a threat IOC feed; can work with an existing feed object or with
                   a new one (by providing all relevant feed parameters).
description:
  - Check if a target can reach or parse a threat IOC feed; can work with an existing feed object or with a new one (by providing all relevant feed
    parameters).
  - All operations are performed over Web Services API.
version_added: "3.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  ioc_feed:
    description:
      - threat ioc feed parameters.
    type: dict
    suboptions:
      name:
        description:
          - Object name.
        type: str
      feed_url:
        description:
          - URL of the feed. URL should be written as http or https.
        type: str
      action:
        description:
          - The feed indicator's action.
        type: str
        choices: ['Prevent', 'Detect']
      certificate_id:
        description:
          - Certificate SHA-1 fingerprint to access the feed.
        type: str
      custom_comment:
        description:
          - Custom IOC feed - the column number of comment.
        type: int
      custom_confidence:
        description:
          - Custom IOC feed - the column number of confidence.
        type: int
      custom_header:
        description:
          - Custom HTTP headers.
        type: list
        elements: dict
        suboptions:
          header_name:
            description:
              - The name of the HTTP header we wish to add.
            type: str
          header_value:
            description:
              - The name of the HTTP value we wish to add.
            type: str
      custom_name:
        description:
          - Custom IOC feed - the column number of name.
        type: int
      custom_severity:
        description:
          - Custom IOC feed - the column number of severity.
        type: int
      custom_type:
        description:
          - Custom IOC feed - the column number of type in case a specific type is not chosen.
        type: int
      custom_value:
        description:
          - Custom IOC feed - the column number of value in case a specific type is chosen.
        type: int
      enabled:
        description:
          - Sets whether this indicator feed is enabled.
        type: bool
      feed_type:
        description:
          - Feed type to be enforced.
        type: str
        choices: ['any type', 'domain', 'ip address', 'md5', 'url', 'ip range', 'mail subject', 'mail from', 'mail to', 'mail reply to',
                 'mail cc', 'sha1', 'sha256']
      password:
        description:
          - password for authenticating with the URL.
        type: str
      use_custom_feed_settings:
        description:
          - Set in order to configure a custom indicator feed.
        type: bool
      username:
        description:
          - username for authenticating with the URL.
        type: str
      fields_delimiter:
        description:
          - The delimiter that separates between the columns in the feed.
        type: str
      ignore_lines_that_start_with:
        description:
          - A prefix that will determine which lines to ignore.
        type: str
      use_gateway_proxy:
        description:
          - Use the gateway's proxy for retrieving the feed.
        type: bool
      details_level:
        description:
          - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
            representation of the object.
        type: str
        choices: ['uid', 'standard', 'full']
      ignore_warnings:
        description:
          - Apply changes ignoring warnings.
        type: bool
      ignore_errors:
        description:
          - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
        type: bool
  targets:
    description:
      - On what targets to execute this command. Targets may be identified by their name, or object unique identifier.
    type: list
    elements: str
  auto_publish_session:
    description:
      - Publish the current session if changes have been performed after task completes.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: check-threat-ioc-feed
  cp_mgmt_check_threat_ioc_feed:
    ioc_feed:
      name: existing_feed
    targets: corporate-gateway
"""

RETURN = """
cp_mgmt_check_threat_ioc_feed:
  description: The checkpoint check-threat-ioc-feed output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_commands,
    api_command,
)


def main():
    argument_spec = dict(
        ioc_feed=dict(
            type="dict",
            options=dict(
                name=dict(type="str"),
                feed_url=dict(type="str"),
                action=dict(type="str", choices=["Prevent", "Detect"]),
                certificate_id=dict(type="str"),
                custom_comment=dict(type="int"),
                custom_confidence=dict(type="int"),
                custom_header=dict(
                    type="list",
                    elements="dict",
                    options=dict(
                        header_name=dict(type="str"),
                        header_value=dict(type="str"),
                    ),
                ),
                custom_name=dict(type="int"),
                custom_severity=dict(type="int"),
                custom_type=dict(type="int"),
                custom_value=dict(type="int"),
                enabled=dict(type="bool"),
                feed_type=dict(
                    type="str",
                    choices=[
                        "any type",
                        "domain",
                        "ip address",
                        "md5",
                        "url",
                        "ip range",
                        "mail subject",
                        "mail from",
                        "mail to",
                        "mail reply to",
                        "mail cc",
                        "sha1",
                        "sha256",
                    ],
                ),
                password=dict(type="str", no_log=True),
                use_custom_feed_settings=dict(type="bool"),
                username=dict(type="str"),
                fields_delimiter=dict(type="str"),
                ignore_lines_that_start_with=dict(type="str"),
                use_gateway_proxy=dict(type="bool"),
                details_level=dict(
                    type="str", choices=["uid", "standard", "full"]
                ),
                ignore_warnings=dict(type="bool"),
                ignore_errors=dict(type="bool"),
            ),
        ),
        targets=dict(type="list", elements="str"),
        auto_publish_session=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "check-threat-ioc-feed"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Copyright (c), Michael DeHaan <michael.dehaan@gmail.com>, 2012-2013
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
A set of helper functions designed to help with initializing boto3/botocore
connections.
"""

from __future__ import annotations

import json
import os
import traceback
import typing
from typing import Any
from typing import Callable
from typing import Dict
from typing import List
from typing import Optional
from typing import Set
from typing import Tuple
from typing import Type
from typing import Union

BOTO3_IMP_ERR = None
try:
    import boto3
    import botocore

    HAS_BOTO3 = True
except ImportError:
    BOTO3_IMP_ERR = traceback.format_exc()
    HAS_BOTO3 = False

if typing.TYPE_CHECKING:
    ClientType = botocore.client.BaseClient
    ResourceType = boto3.resources.base.ServiceResource
    BotoConn = Union[ClientType, ResourceType, Tuple[ClientType, ResourceType]]
    from .modules import AnsibleAWSModule

try:
    from packaging.version import Version

    HAS_PACKAGING = True
except ImportError:
    HAS_PACKAGING = False

from ansible.module_utils._text import to_native
from ansible.module_utils.ansible_release import __version__
from ansible.module_utils.basic import missing_required_lib
from ansible.module_utils.six import binary_type
from ansible.module_utils.six import text_type

from .common import get_collection_info
from .exceptions import AnsibleBotocoreError
from .retries import AWSRetry

MINIMUM_BOTOCORE_VERSION = "1.31.0"
MINIMUM_BOTO3_VERSION = "1.28.0"


def _get_user_agent_string():
    info = get_collection_info()
    result = f"APN/1.0 Ansible/{__version__}"
    if info["name"]:
        if info["version"] is not None:
            result += f" {info['name']}/{info['version']}"
        else:
            result += f" {info['name']}"
    return result


def boto3_conn(
    module: AnsibleAWSModule,
    conn_type: str,
    resource: Optional[str] = None,
    region: Optional[str] = None,
    endpoint: Optional[str] = None,
    **params,
) -> BotoConn:
    """
    Builds a boto3 resource/client connection cleanly wrapping the most common failures.
    Handles:
        ValueError, botocore.exceptions.BotoCoreError
    """
    try:
        return _boto3_conn(conn_type=conn_type, resource=resource, region=region, endpoint=endpoint, **params)
    except botocore.exceptions.NoRegionError:
        module.fail_json(
            msg=f"The {module._name} module requires a region and none was found in configuration, "
            "environment variables or module parameters",
        )
    except (ValueError, botocore.exceptions.BotoCoreError) as e:
        if hasattr(module, "fail_json_aws"):
            module.fail_json_aws(e, msg="Couldn't connect to AWS")
        module.fail_json(msg=f"Couldn't connect to AWS: {to_native(e)}")


def _merge_botocore_config(
    config_a: botocore.config.Config,
    config_b: Union[botocore.config.Config, Dict],
) -> botocore.config.Config:
    """
    Merges the extra configuration options from config_b into config_a.
    Supports both botocore.config.Config objects and dicts
    """
    if not config_b:
        return config_a
    if not isinstance(config_b, botocore.config.Config):
        config_b = botocore.config.Config(**config_b)
    return config_a.merge(config_b)


def _boto3_conn(
    conn_type: str,
    resource: Optional[str] = None,
    region: Optional[str] = None,
    endpoint: Optional[str] = None,
    **params,
) -> BotoConn:
    """
    Builds a boto3 resource/client connection cleanly wrapping the most common failures.
    No exceptions are caught/handled.
    """
    profile = params.pop("profile_name", None)

    if conn_type not in ["both", "resource", "client"]:
        raise ValueError(
            "There is an issue in the calling code. You "
            "must specify either both, resource, or client to "
            "the conn_type parameter in the boto3_conn function "
            "call"
        )

    # default config with user agent
    config = botocore.config.Config(
        user_agent=_get_user_agent_string(),
    )

    for param in ("config", "aws_config"):
        config = _merge_botocore_config(config, params.pop(param, None))

    session = boto3.session.Session(
        profile_name=profile,
    )

    enable_placebo(session)

    if conn_type == "resource":
        return session.resource(resource, config=config, region_name=region, endpoint_url=endpoint, **params)
    elif conn_type == "client":
        return session.client(resource, config=config, region_name=region, endpoint_url=endpoint, **params)
    else:
        client = session.client(resource, config=config, region_name=region, endpoint_url=endpoint, **params)
        boto3_resource = session.resource(resource, config=config, region_name=region, endpoint_url=endpoint, **params)
        return client, boto3_resource


# Inventory plugins don't have access to the same 'module', they need to throw
# an exception rather than calling module.fail_json
boto3_inventory_conn = _boto3_conn


def boto_exception(err: Exception) -> str:
    """
    Extracts the error message from a boto exception.

    :param err: Exception from boto
    :return: Error message
    """
    if hasattr(err, "error_message"):
        error = err.error_message  # pyright: ignore[reportAttributeAccessIssue]
    elif hasattr(err, "message"):
        error = (
            str(err.message) + " " + str(err) + " - " + str(type(err))  # pyright: ignore[reportAttributeAccessIssue]
        )
    else:
        error = f"{Exception}: {err}"

    return error


def _aws_region(params: Dict) -> Optional[str]:
    region = params.get("region")

    if region:
        return region

    if not HAS_BOTO3:
        raise AnsibleBotocoreError(message=missing_required_lib("boto3 and botocore"), exception=BOTO3_IMP_ERR)

    # here we don't need to make an additional call, will default to 'us-east-1' if the below evaluates to None.
    try:
        # Botocore doesn't like empty strings, make sure we default to None in the case of an empty
        # string.
        profile_name = params.get("profile") or None
        return boto3.session.Session(profile_name=profile_name).region_name
    except botocore.exceptions.BotoCoreError:
        return None


def get_aws_region(
    module: AnsibleAWSModule,
    boto3: Optional[bool] = None,
) -> Optional[str]:  # pylint: disable=redefined-outer-name
    if boto3 is not None:
        module.deprecate(
            "get_aws_region(): the boto3 parameter will be removed in a release after 2025-05-01. "
            "The parameter has been ignored since release 4.0.0.",
            date="2025-05-01",
            collection_name="amazon.aws",
        )
    try:
        return _aws_region(module.params)
    except AnsibleBotocoreError as e:
        if e.exception:
            module.fail_json(msg=e.message, exception=e.exception)
        else:
            module.fail_json(msg=e.message)


def _aws_connection_info(params: Dict) -> Tuple[Optional[str], Optional[str], Dict[str, Any]]:
    endpoint_url = params.get("endpoint_url")
    access_key = params.get("access_key")
    secret_key = params.get("secret_key")
    session_token = params.get("session_token")
    region = _aws_region(params)
    profile_name = params.get("profile")
    validate_certs = params.get("validate_certs")
    ca_bundle = params.get("aws_ca_bundle")
    config = params.get("aws_config")

    # Caught here so that they can be deliberately set to '' to avoid conflicts when environment
    # variables are also being used
    if profile_name and (access_key or secret_key or session_token):
        raise AnsibleBotocoreError(message="Passing both a profile and access tokens is not supported.")

    # Botocore doesn't like empty strings, make sure we default to None in the case of an empty
    # string.
    if not access_key:
        access_key = None
    if not secret_key:
        secret_key = None
    if not session_token:
        session_token = None

    if profile_name:
        boto_params = dict(
            aws_access_key_id=None,
            aws_secret_access_key=None,
            aws_session_token=None,
            profile_name=profile_name,
        )
    else:
        boto_params = dict(
            aws_access_key_id=access_key,
            aws_secret_access_key=secret_key,
            aws_session_token=session_token,
        )

    if validate_certs and ca_bundle:
        boto_params["verify"] = ca_bundle
    else:
        boto_params["verify"] = validate_certs

    if config is not None:
        boto_params["aws_config"] = botocore.config.Config(**config)

    for param, value in boto_params.items():
        if isinstance(value, binary_type):
            boto_params[param] = text_type(value, "utf-8", "strict")

    return region, endpoint_url, boto_params


def get_aws_connection_info(
    module: AnsibleAWSModule,
    boto3: Optional[bool] = None,
) -> Tuple[Optional[str], Optional[str], Dict]:  # pylint: disable=redefined-outer-name
    if boto3 is not None:
        module.deprecate(
            "get_aws_connection_info(): the boto3 parameter will be removed in a release after 2025-05-01. "
            "The parameter has been ignored since release 4.0.0.",
            date="2025-05-01",
            collection_name="amazon.aws",
        )
    try:
        return _aws_connection_info(module.params)
    except AnsibleBotocoreError as e:
        if e.exception:
            module.fail_json(msg=e.message, exception=e.exception)
        else:
            module.fail_json(msg=e.message)


def _paginated_query(
    client: ClientType,
    paginator_name: str,
    **params,
) -> Any:
    paginator = client.get_paginator(paginator_name)
    result = paginator.paginate(**params).build_full_result()
    return result


def paginated_query_with_retries(
    client: ClientType,
    paginator_name: str,
    retry_decorator=None,
    **params,
) -> Any:
    """
    Performs a boto3 paginated query.
    By default uses uses AWSRetry.jittered_backoff(retries=10) to retry queries
    with temporary failures.

    Examples:
        tags = paginated_query_with_retries(client, "describe_tags", Filters=[])

        decorator = AWSRetry.backoff(tries=5, delay=5, backoff=2.0,
                                     catch_extra_error_codes=['RequestInProgressException'])
        certificates = paginated_query_with_retries(client, "list_certificates", decorator)
    """
    if retry_decorator is None:
        retry_decorator = AWSRetry.jittered_backoff(retries=10)
    result = retry_decorator(_paginated_query)(client, paginator_name, **params)
    return result


def gather_sdk_versions() -> Dict:
    """Gather AWS SDK (boto3 and botocore) dependency versions

    Returns {'boto3_version': str, 'botocore_version': str}
    Returns {} if either module is not installed
    """
    if not HAS_BOTO3:
        return {}

    return dict(
        boto3_version=boto3.__version__,
        botocore_version=botocore.__version__,
    )


def is_boto3_error_code(
    code: Union[List, Tuple, Set, str],
    e: Optional[BaseException] = None,
) -> Type[Exception]:
    """Check if the botocore exception is raised by a specific error code.

    Returns ClientError if the error code matches, a dummy exception if it does not have an error code or does not match

    Example:
    try:
        ec2.describe_instances(InstanceIds=['potato'])
    except is_boto3_error_code('InvalidInstanceID.Malformed'):
        # handle the error for that code case
    except botocore.exceptions.ClientError as e:
        # handle the generic error case for all other codes
    """
    from botocore.exceptions import ClientError

    if e is None:
        import sys

        dummy, e, dummy = sys.exc_info()
    if not isinstance(code, (list, tuple, set)):
        code = [code]
    if isinstance(e, ClientError) and e.response["Error"]["Code"] in code:
        return ClientError
    return type("NeverEverRaisedException", (Exception,), {})


def is_boto3_error_message(
    msg: str,
    e: Optional[BaseException] = None,
) -> Type[Exception]:
    """Check if the botocore exception contains a specific error message.

    Returns ClientError if the error code matches, a dummy exception if it does not have an error code or does not match

    Example:
    try:
        ec2.describe_vpc_classic_link(VpcIds=[vpc_id])
    except is_boto3_error_message('The functionality you requested is not available in this region.'):
        # handle the error for that error message
    except botocore.exceptions.ClientError as e:
        # handle the generic error case for all other codes
    """
    from botocore.exceptions import ClientError

    if e is None:
        import sys

        dummy, e, dummy = sys.exc_info()
    if isinstance(e, ClientError) and msg in e.response["Error"]["Message"]:
        return ClientError
    return type("NeverEverRaisedException", (Exception,), {})


def get_boto3_client_method_parameters(
    client: ClientType,
    method_name: str,
    required=False,
) -> List:
    op = client.meta.method_to_api_mapping.get(method_name)
    input_shape = client._service_model.operation_model(op).input_shape
    if not input_shape:
        parameters = []
    elif required:
        parameters = list(input_shape.required_members)
    else:
        parameters = list(input_shape.members.keys())
    return parameters


# Used by normalize_boto3_result
def _boto3_handler(obj: Any) -> Any:
    if hasattr(obj, "isoformat"):
        return obj.isoformat()
    else:
        return obj


def normalize_boto3_result(
    result: Union[List, Dict, Set],
) -> Union[List, Dict, Set]:
    """
    Because Boto3 returns datetime objects where it knows things are supposed to
    be dates we need to mass-convert them over to strings which Ansible/Jinja
    handle better.  This also makes it easier to compare complex objects which
    include a mix of dates in string format (from parameters) and dates as
    datetime objects.  Boto3 is happy to be passed ISO8601 format strings.
    """
    return json.loads(json.dumps(result, default=_boto3_handler))


def enable_placebo(session: boto3.session.Session) -> None:
    """
    Helper to record or replay offline modules for testing purpose.
    """
    if "_ANSIBLE_PLACEBO_RECORD" in os.environ:
        import placebo

        existing_entries = os.listdir(os.environ["_ANSIBLE_PLACEBO_RECORD"])
        idx = len(existing_entries)
        data_path = f"{os.environ['_ANSIBLE_PLACEBO_RECORD']}/{idx}"
        os.mkdir(data_path)
        pill = placebo.attach(session, data_path=data_path)
        pill.record()
    if "_ANSIBLE_PLACEBO_REPLAY" in os.environ:
        import shutil

        import placebo

        existing_entries = sorted([int(i) for i in os.listdir(os.environ["_ANSIBLE_PLACEBO_REPLAY"])])
        idx = str(existing_entries[0])
        data_path = os.environ["_ANSIBLE_PLACEBO_REPLAY"] + "/" + idx
        try:
            shutil.rmtree("_tmp")
        except FileNotFoundError:
            pass
        shutil.move(data_path, "_tmp")
        if len(existing_entries) == 1:
            os.rmdir(os.environ["_ANSIBLE_PLACEBO_REPLAY"])
        pill = placebo.attach(session, data_path="_tmp")
        pill.playback()


def check_sdk_version_supported(
    botocore_version: Optional[str] = None,
    boto3_version: Optional[str] = None,
    warn: Optional[Callable] = None,
) -> bool:
    """Checks to see if the available boto3 / botocore versions are supported
    args:
        botocore_version: (str) overrides the minimum version of botocore supported by the collection
        boto3_version: (str) overrides the minimum version of boto3 supported by the collection
        warn: (Callable) invoked with a string message if boto3/botocore are less than the
            supported versions
    raises:
        AnsibleBotocoreError - If botocore/boto3 is missing
    returns
        False if boto3 or botocore is less than the minimum supported versions
        True if boto3 and botocore are greater than or equal the the minimum supported versions
    """

    botocore_version = botocore_version or MINIMUM_BOTOCORE_VERSION
    boto3_version = boto3_version or MINIMUM_BOTO3_VERSION

    if not HAS_BOTO3:
        raise AnsibleBotocoreError(message=missing_required_lib("botocore and boto3"))

    supported = True

    if not HAS_PACKAGING:
        if warn:
            warn("packaging.version Python module not installed, unable to check AWS SDK versions")
        return True
    if not botocore_at_least(botocore_version):
        supported = False
        if warn:
            warn(f"botocore < {MINIMUM_BOTOCORE_VERSION} is not supported or tested.  Some features may not work.")
    if not boto3_at_least(boto3_version):
        supported = False
        if warn:
            warn(f"boto3 < {MINIMUM_BOTO3_VERSION} is not supported or tested.  Some features may not work.")

    return supported


def _version_at_least(a: str, b: str) -> bool:
    if not HAS_PACKAGING:
        return True
    return Version(a) >= Version(b)


def boto3_at_least(desired: str) -> bool:
    """Check if the available boto3 version is greater than or equal to a desired version.

    Usage:
        if module.params.get('assign_ipv6_address') and not module.boto3_at_least('1.4.4'):
            # conditionally fail on old boto3 versions if a specific feature is not supported
            module.fail_json(msg="Boto3 can't deal with EC2 IPv6 addresses before version 1.4.4.")
    """
    existing = gather_sdk_versions()
    return _version_at_least(existing["boto3_version"], desired)


def botocore_at_least(desired: str) -> bool:
    """Check if the available botocore version is greater than or equal to a desired version.

    Usage:
        if not module.botocore_at_least('1.2.3'):
            module.fail_json(msg='The Serverless Elastic Load Compute Service is not in botocore before v1.2.3')
        if not module.botocore_at_least('1.5.3'):
            module.warn('Botocore did not include waiters for Service X before 1.5.3. '
                        'To wait until Service X resources are fully available, update botocore.')
    """
    existing = gather_sdk_versions()
    return _version_at_least(existing["botocore_version"], desired)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: network_devices_count_info
short_description: Information module for Network Devices Count Info
description:
- This module represents an alias of the module network_devices_count_v1_info
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  startTime:
    description:
    - >
      StartTime query parameter. Start time from which API queries the data set related to the resource. It must
      be specified in UNIX epochtime in milliseconds. Value is inclusive. If `startTime` is not provided, API will
      default to current time.
    type: float
  endTime:
    description:
    - >
      EndTime query parameter. End time to which API queries the data set related to the resource. It must be
      specified in UNIX epochtime in milliseconds. Value is inclusive.
    type: float
  id:
    description:
    - >
      Id query parameter. The list of entity Uuids. (Ex."6bef213c-19ca-4170-8375-b694e251101c") Examples
      id=6bef213c-19ca-4170-8375-b694e251101c (single entity uuid requested) id=6bef213c-19ca-4170-8375-
      b694e251101c&id=32219612-819e-4b5e-a96b-cf22aca13dd9&id=2541e9a7-b80d-4955-8aa2-79b233318ba0 (multiple
      entity uuid with '&' separator).
    type: str
  siteHierarchy:
    description:
    - >
      SiteHierarchy query parameter. The full hierarchical breakdown of the site tree starting from Global site
      name and ending with the specific site name. The Root site is named "Global" (Ex.
      `Global/AreaName/BuildingName/FloorName`) This field supports wildcard asterisk (*) character search
      support. E.g. */San*, */San, /San* Examples `?siteHierarchy=Global/AreaName/BuildingName/FloorName` (single
      siteHierarchy requested) `?siteHierarchy=Global/AreaName/BuildingName/FloorName&siteHierarchy=Global/AreaNam
      e2/BuildingName2/FloorName2` (multiple siteHierarchies requested).
    type: str
  siteHierarchyId:
    description:
    - >
      SiteHierarchyId query parameter. The full hierarchy breakdown of the site tree in id form starting from
      Global site UUID and ending with the specific site UUID. (Ex. `globalUuid/areaUuid/buildingUuid/floorUuid`)
      This field supports wildcard asterisk (*) character search support. E.g. `*uuid*, *uuid, uuid* Examples
      `?siteHierarchyId=globalUuid/areaUuid/buildingUuid/floorUuid `(single siteHierarchyId requested) `?siteHiera
      rchyId=globalUuid/areaUuid/buildingUuid/floorUuid&siteHierarchyId=globalUuid/areaUuid2/buildingUuid2/floorUu
      id2` (multiple siteHierarchyIds requested).
    type: str
  siteId:
    description:
    - >
      SiteId query parameter. The UUID of the site. (Ex. `flooruuid`) This field supports wildcard asterisk (*)
      character search support. E.g.*flooruuid*, *flooruuid, flooruuid* Examples `?siteId=id1` (single id
      requested) `?siteId=id1&siteId=id2&siteId=id3` (multiple ids requested).
    type: str
  managementIpAddress:
    description:
    - >
      ManagementIpAddress query parameter. The list of entity management IP Address. It can be either Ipv4 or Ipv6
      address or combination of both(Ex. "121.1.1.10") This field supports wildcard (`*`) character-based search.
      Ex `*1.1*` or `1.1*` or `*1.1` Examples managementIpAddresses=121.1.1.10
      managementIpAddresses=121.1.1.10&managementIpAddresses=172.20.1.10&managementIpAddresses=200
      10&=managementIpAddresses172.20.3.4 (multiple entity IP Address with & separator).
    type: str
  macAddress:
    description:
    - >
      MacAddress query parameter. The macAddress of the network device or client This field supports wildcard
      (`*`) character-based search. Ex `*AB AB AB*` or `AB AB AB*` or `*AB AB AB` Examples `macAddress=AB AB AB CD
      CD CD` (single macAddress requested) `macAddress=AB AB AB CD CD DC&macAddress=AB AB AB CD CD FE` (multiple
      macAddress requested).
    type: str
  family:
    description:
    - >
      Family query parameter. The list of network device family names Examples family=Switches and Hubs (single
      network device family name )family=Switches and Hubs&family=Router&family=Wireless Controller (multiple
      Network device family names with & separator). This field is not case sensitive.
    type: str
  type:
    description:
    - >
      Type query parameter. The list of network device type This field supports wildcard (`*`) character-based
      search. Ex `*9407R*` or `*9407R` or `9407R*`Examples type=SwitchesCisco Catalyst 9407R Switch (single
      network device types )type=Cisco Catalyst 38xx stack-able ethernet switch&type=Cisco 3945 Integrated
      Services Router G2 (multiple Network device types with & separator).
    type: str
  role:
    description:
    - >
      Role query parameter. The list of network device role. Examples role=CORE, role=CORE&role=ACCESS&role=ROUTER
      (multiple Network device roles with & separator). This field is not case sensitive.
    type: str
  serialNumber:
    description:
    - >
      SerialNumber query parameter. The list of network device serial numbers. This field supports wildcard (`*`)
      character-based search. Ex `*MS1SV*` or `MS1SV*` or `*MS1SV` Examples serialNumber=9FUFMS1SVAX
      serialNumber=9FUFMS1SVAX&FCW2333Q0BY&FJC240617JX(multiple Network device serial number with & separator).
    type: str
  maintenanceMode:
    description:
    - MaintenanceMode query parameter. The device maintenanceMode status true or false.
    type: bool
  softwareVersion:
    description:
    - >
      SoftwareVersion query parameter. The list of network device software version This field supports wildcard
      (`*`) character-based search. Ex `*17.8*` or `*17.8` or `17.8*` Examples softwareVersion=2.3.4.0 (single
      network device software version ) softwareVersion=17.9.3.23&softwareVersion=17.7.1.2&softwareVersion=*.17.7
      (multiple Network device software versions with & separator).
    type: str
  healthScore:
    description:
    - >
      HealthScore query parameter. The list of entity health score categories Examples
      healthScore=good,healthScore=good&healthScore=fair (multiple entity healthscore values with & separator).
      This field is not case sensitive.
    type: str
  view:
    description:
    - >
      View query parameter. The List of Network Device model views. Please refer to ```NetworkDeviceView``` for
      the supported list.
    type: str
  attribute:
    description:
    - >
      Attribute query parameter. The List of Network Device model attributes. This is helps to specify the
      interested fields in the request.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices GetsTheTotalNetworkDeviceCountsBasedOnTheProvidedQueryParametersV1
  description: Complete reference of the GetsTheTotalNetworkDeviceCountsBasedOnTheProvidedQueryParametersV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!gets-the-total-network-device-counts-based-on-the-provided-query-parameters # noqa: E501
notes:
  - SDK Method used are
    devices.Devices.gets_the_total_network_device_counts_based_on_the_provided_query_parameters_v1,

  - Paths used are
    get /dna/data/api/v1/networkDevices/count,
  - It should be noted that this module is an alias of network_devices_count_v1_info

"""

EXAMPLES = r"""
- name: Get all Network Devices Count Info
  cisco.dnac.network_devices_count_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    startTime: 0
    endTime: 0
    id: string
    siteHierarchy: string
    siteHierarchyId: string
    siteId: string
    managementIpAddress: string
    macAddress: string
    family: string
    type: string
    role: string
    serialNumber: string
    maintenanceMode: True
    softwareVersion: string
    healthScore: string
    view: string
    attribute: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: endpoint_analytics_endpoints_anc_policy_delete_v1
short_description: Resource module for Endpoint Analytics Endpoints Anc Policy Delete V1
description:
- Manage operation delete of the resource Endpoint Analytics Endpoints Anc Policy Delete V1.
- Revokes given ANC policy from the endpoint.
version_added: '6.16.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  epId:
    description: EpId path parameter. Unique identifier for the endpoint.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for AI Endpoint Analytics RevokeANCPolicyV1
  description: Complete reference of the RevokeANCPolicyV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!revoke-anc-policy
notes:
  - SDK Method used are
    ai_endpoint_analytics.AIEndpointAnalytics.revoke_anc_policy_v1,

  - Paths used are
    delete /dna/intent/api/v1/endpoint-analytics/endpoints/{epId}/anc-policy,

"""

EXAMPLES = r"""
- name: Delete all
  cisco.dnac.endpoint_analytics_endpoints_anc_policy_delete_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    epId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_devices_v1_info
short_description: Information module for Network Devices V1
description:
- Get all Network Devices V1.
- Get Network Devices V1 by id.
- >
   Gets the Network Device details based on the provided query parameters. When there is no start and end time
   specified returns the latest device details. For detailed information about the usage of the API, please refer to
   the Open API specification document - https //github.com/cisco-en-programmability/catalyst-center-api-
   specs/blob/main/Assurance/CE_Cat_Center_Org-AssuranceNetworkDevices-1.0.2-resolved.yaml.
- >
   Returns the device data for the given device Uuid in the specified start and end time range. When there is no
   start and end time specified returns the latest available data for the given Id. For detailed information about
   the usage of the API, please refer to the Open API specification document - https //github.com/cisco-en-
   programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-
   AssuranceNetworkDevices-1.0.2-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  startTime:
    description:
    - >
      StartTime query parameter. Start time from which API queries the data set related to the resource. It must
      be specified in UNIX epochtime in milliseconds. Value is inclusive. If `startTime` is not provided, API will
      default to current time.
    type: float
  endTime:
    description:
    - >
      EndTime query parameter. End time to which API queries the data set related to the resource. It must be
      specified in UNIX epochtime in milliseconds. Value is inclusive.
    type: float
  limit:
    description:
    - Limit query parameter. Maximum number of records to return.
    type: float
  offset:
    description:
    - >
      Offset query parameter. Specifies the starting point within all records returned by the API. It's one based
      offset. The starting value is 1.
    type: float
  sortBy:
    description:
    - SortBy query parameter. A field within the response to sort by.
    type: str
  order:
    description:
    - Order query parameter. The sort order of the field ascending or descending.
    type: str
  siteHierarchy:
    description:
    - >
      SiteHierarchy query parameter. The full hierarchical breakdown of the site tree starting from Global site
      name and ending with the specific site name. The Root site is named "Global" (Ex.
      `Global/AreaName/BuildingName/FloorName`) This field supports wildcard asterisk (*) character search
      support. E.g. */San*, */San, /San* Examples `?siteHierarchy=Global/AreaName/BuildingName/FloorName` (single
      siteHierarchy requested) `?siteHierarchy=Global/AreaName/BuildingName/FloorName&siteHierarchy=Global/AreaNam
      e2/BuildingName2/FloorName2` (multiple siteHierarchies requested).
    type: str
  siteHierarchyId:
    description:
    - >
      SiteHierarchyId query parameter. The full hierarchy breakdown of the site tree in id form starting from
      Global site UUID and ending with the specific site UUID. (Ex. `globalUuid/areaUuid/buildingUuid/floorUuid`)
      This field supports wildcard asterisk (*) character search support. E.g. `*uuid*, *uuid, uuid* Examples
      `?siteHierarchyId=globalUuid/areaUuid/buildingUuid/floorUuid `(single siteHierarchyId requested) `?siteHiera
      rchyId=globalUuid/areaUuid/buildingUuid/floorUuid&siteHierarchyId=globalUuid/areaUuid2/buildingUuid2/floorUu
      id2` (multiple siteHierarchyIds requested).
    type: str
  siteId:
    description:
    - >
      SiteId query parameter. The UUID of the site. (Ex. `flooruuid`) This field supports wildcard asterisk (*)
      character search support. E.g.*flooruuid*, *flooruuid, flooruuid* Examples `?siteId=id1` (single id
      requested) `?siteId=id1&siteId=id2&siteId=id3` (multiple ids requested).
    type: str
  id:
    description:
    - >
      Id query parameter. The list of entity Uuids. (Ex."6bef213c-19ca-4170-8375-b694e251101c") Examples
      id=6bef213c-19ca-4170-8375-b694e251101c (single entity uuid requested) id=6bef213c-19ca-4170-8375-
      b694e251101c&id=32219612-819e-4b5e-a96b-cf22aca13dd9&id=2541e9a7-b80d-4955-8aa2-79b233318ba0 (multiple
      entity uuid with '&' separator).
    type: str
  managementIpAddress:
    description:
    - >
      ManagementIpAddress query parameter. The list of entity management IP Address. It can be either Ipv4 or Ipv6
      address or combination of both(Ex. "121.1.1.10") This field supports wildcard (`*`) character-based search.
      Ex `*1.1*` or `1.1*` or `*1.1` Examples managementIpAddresses=121.1.1.10
      managementIpAddresses=121.1.1.10&managementIpAddresses=172.20.1.10&managementIpAddresses=200
      10&=managementIpAddresses172.20.3.4 (multiple entity IP Address with & separator).
    type: str
  macAddress:
    description:
    - >
      MacAddress query parameter. The macAddress of the network device or client This field supports wildcard
      (`*`) character-based search. Ex `*AB AB AB*` or `AB AB AB*` or `*AB AB AB` Examples `macAddress=AB AB AB CD
      CD CD` (single macAddress requested) `macAddress=AB AB AB CD CD DC&macAddress=AB AB AB CD CD FE` (multiple
      macAddress requested).
    type: str
  family:
    description:
    - >
      Family query parameter. The list of network device family names Examples family=Switches and Hubs (single
      network device family name )family=Switches and Hubs&family=Router&family=Wireless Controller (multiple
      Network device family names with & separator). This field is not case sensitive.
    type: str
  type:
    description:
    - >
      Type query parameter. The list of network device type This field supports wildcard (`*`) character-based
      search. Ex `*9407R*` or `*9407R` or `9407R*` Examples type=SwitchesCisco Catalyst 9407R Switch (single
      network device types ) type=Cisco Catalyst 38xx stack-able ethernet switch&type=Cisco 3945 Integrated
      Services Router G2 (multiple Network device types with & separator).
    type: str
  role:
    description:
    - >
      Role query parameter. The list of network device role. Examples role=CORE, role=CORE&role=ACCESS&role=ROUTER
      (multiple Network device roles with & separator). This field is not case sensitive.
    type: str
  serialNumber:
    description:
    - >
      SerialNumber query parameter. The list of network device serial numbers. This field supports wildcard (`*`)
      character-based search. Ex `*MS1SV*` or `MS1SV*` or `*MS1SV` Examples serialNumber=9FUFMS1SVAX
      serialNumber=9FUFMS1SVAX&FCW2333Q0BY&FJC240617JX(multiple Network device serial number with & separator).
    type: str
  maintenanceMode:
    description:
    - MaintenanceMode query parameter. The device maintenanceMode status true or false.
    type: bool
  softwareVersion:
    description:
    - >
      SoftwareVersion query parameter. The list of network device software version This field supports wildcard
      (`*`) character-based search. Ex `*17.8*` or `*17.8` or `17.8*` Examples softwareVersion=2.3.4.0 (single
      network device software version ) softwareVersion=17.9.3.23&softwareVersion=17.7.1.2&softwareVersion=*.17.7
      (multiple Network device software versions with & separator).
    type: str
  healthScore:
    description:
    - >
      HealthScore query parameter. The list of entity health score categories Examples healthScore=good,
      healthScore=good&healthScore=fair (multiple entity healthscore values with & separator). This field is not
      case sensitive.
    type: str
  view:
    description:
    - >
      View query parameter. The List of Network Device model views. Please refer to ```NetworkDeviceView``` for
      the supported list.
    type: str
  attribute:
    description:
    - >
      Attribute query parameter. The List of Network Device model attributes. This is helps to specify the
      interested fields in the request.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices GetTheDeviceDataForTheGivenDeviceIdUuidV1
  description: Complete reference of the GetTheDeviceDataForTheGivenDeviceIdUuidV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-the-device-data-for-the-given-device-id-uuid
- name: Cisco DNA Center documentation for Devices GetsTheNetworkDeviceDetailsBasedOnTheProvidedQueryParametersV1
  description: Complete reference of the GetsTheNetworkDeviceDetailsBasedOnTheProvidedQueryParametersV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!gets-the-network-device-details-based-on-the-provided-query-parameters  # noqa: E501
notes:
  - SDK Method used are
    devices.Devices.get_the_device_data_for_the_given_device_id_uuid_v1,
    devices.Devices.gets_the_network_device_details_based_on_the_provided_query_parameters_v1,

  - Paths used are
    get /dna/data/api/v1/networkDevices,
    get /dna/data/api/v1/networkDevices/{id},

"""

EXAMPLES = r"""
- name: Get all Network Devices V1
  cisco.dnac.network_devices_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    startTime: 0
    endTime: 0
    limit: 0
    offset: 0
    sortBy: string
    order: string
    siteHierarchy: string
    siteHierarchyId: string
    siteId: string
    id: string
    managementIpAddress: string
    macAddress: string
    family: string
    type: string
    role: string
    serialNumber: string
    maintenanceMode: True
    softwareVersion: string
    healthScore: string
    view: string
    attribute: string
  register: result

- name: Get Network Devices V1 by id
  cisco.dnac.network_devices_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    startTime: 0
    endTime: 0
    view: string
    attribute: string
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "id": "string",
        "name": "string",
        "managementIpAddress": "string",
        "platformId": "string",
        "deviceFamily": "string",
        "serialNumber": "string",
        "macAddress": "string",
        "deviceSeries": "string",
        "softwareVersion": "string",
        "productVendor": "string",
        "deviceRole": "string",
        "deviceType": "string",
        "communicationState": "string",
        "collectionStatus": "string",
        "haStatus": "string",
        "lastBootTime": 0,
        "siteHierarchyId": "string",
        "siteHierarchy": "string",
        "siteId": "string",
        "deviceGroupHierarchyId": "string",
        "tagNames": [
          "string"
        ],
        "stackType": "string",
        "osType": "string",
        "ringStatus": true,
        "maintenanceModeEnabled": true,
        "upTime": 0,
        "ipv4Address": "string",
        "ipv6Address": "string",
        "redundancyMode": "string",
        "featureFlagList": [
          "string"
        ],
        "haLastResetReason": "string",
        "redundancyPeerStateDerived": "string",
        "redundancyPeerState": "string",
        "redundancyStateDerived": "string",
        "redundancyState": "string",
        "wiredClientCount": 0,
        "wirelessClientCount": 0,
        "portCount": 0,
        "clientCount": 0,
        "apDetails": {
          "connectedWlcName": "string",
          "policyTagName": "string",
          "apOperationalState": "string",
          "powerSaveMode": "string",
          "operationalMode": "string",
          "resetReason": "string",
          "protocol": "string",
          "powerMode": "string",
          "connectedTime": 0,
          "ledFlashEnabled": true,
          "ledFlashSeconds": 0,
          "subMode": "string",
          "homeApEnabled": true,
          "powerType": "string",
          "apType": "string",
          "adminState": "string",
          "icapCapability": "string",
          "regulatoryDomain": "string",
          "ethernetMac": "string",
          "rfTagName": "string",
          "siteTagName": "string",
          "powerSaveModeCapable": "string",
          "powerProfile": "string",
          "flexGroup": "string",
          "powerCalendarProfile": "string",
          "apGroup": "string",
          "radios": [
            {
              "id": "string",
              "band": "string",
              "noise": 0,
              "airQuality": 0,
              "interference": 0,
              "trafficUtil": 0,
              "utilization": 0,
              "clientCount": 0
            }
          ]
        },
        "metricsDetails": {
          "overallHealthScore": 0,
          "overallFabricScore": 0,
          "cpuUtilization": 0,
          "cpuScore": 0,
          "memoryUtilization": 0,
          "memoryScore": 0,
          "avgTemperature": 0,
          "maxTemperature": 0,
          "discardScore": 0,
          "discardInterfaces": [
            "string"
          ],
          "errorScore": 0,
          "errorInterfaces": [
            "string"
          ],
          "interDeviceLinkScore": 0,
          "interDeviceConnectedDownInterfaces": [
            "string"
          ],
          "linkUtilizationScore": 0,
          "highLinkUtilizationInterfaces": [
            "string"
          ],
          "freeTimerScore": 0,
          "freeTimer": 0,
          "packetPoolScore": 0,
          "packetPool": 0,
          "freeMemoryBufferScore": 0,
          "freeMemoryBuffer": 0,
          "wqePoolScore": 0,
          "wqePool": 0,
          "apCount": 0,
          "noiseScore": 0,
          "utilizationScore": 0,
          "interferenceScore": 0,
          "airQualityScore": 0
        },
        "fabricDetails": {
          "fabricRole": [
            "string"
          ],
          "fabricSiteName": "string",
          "transitFabrics": [
            "string"
          ]
        },
        "aggregateAttributes": [
          {
            "name": "string",
            "function": "string",
            "value": 0
          }
        ]
      },
      "version": "string"
    }
"""
